% APM Web-Interface Commands
% Report model details such as values, multipliers, residuals, 
%   first derivatives, and second derivatives
function y = apm_details(server,app,x,lam)

% retrieve values so they can be restored later
max_iter = apm_tag(server,app,'nlc.max_iter');
diaglevel = apm_tag(server,app,'nlc.diaglevel');

% zero iterations - return stats only
apm_option(server,app,'nlc.max_iter',0);

% diaglevel must be at least 2
apm_option(server,app,'nlc.diaglevel',2);

% optionally load solution vector of variables
if (nargin>=3),
    % create 'warm' as a column vector
    if (size(x,1)==1),
        warm = x';
    else
        warm = x;
    end
    save -ascii 'warm.t0' warm
    t0_load(server,app,'warm.t0');
    delete('warm.t0');
end

% optionally load lagrange multipliers to server
if (nargin>=4),
    % create 'lam' as a column vector
    if (size(lam,1)==1),
        lam = lam';
    end
    save -ascii 'lam.t0' lam
    t0_load(server,app,'lam.t0');
    delete('lam.t0');
end

% compute on server
output = apm(server,app,'solve');

% retrieve sol and solution.csv
sol = apm_sol(server,app);
delete('solution.csv');
% initialize y (structure to store problem information)
y = [];
% extract names
y.names = sol(1,:);
% extract values
cc = cell2mat(sol(2:end,:));
% generate variable names as a structure
for i = 1:size(cc,2);
    % remove some offending characters for MATLAB that
    %   are allowed in APM including: "[],"
    var_name = strrep(y.names(i), '[', '_');
    var_name = strrep(var_name, ']', '');
    var_name = strrep(var_name, ',', '_');
    eval(['y.table.' y.names{i} '= cc(:,' int2str(i) ');']);
end

% retrieve variables and bounds
apm_get(server,app,'apm_var.txt');
load apm_var.txt
delete('apm_var.txt');
y.nvar = size(apm_var,1); % get number of variables
y.var.lb = apm_var(:,1);
y.var.val = apm_var(:,2);
y.var.ub = apm_var(:,3);

% retrieve objective function value
apm_get(server,app,'apm_obj.txt');
load apm_obj.txt
delete('apm_obj.txt');
y.obj = apm_obj;

% retrieve objective gradient
apm_get(server,app,'apm_obj_grad.txt');
load apm_obj_grad.txt
delete('apm_obj_grad.txt');
y.obj_grad = apm_obj_grad;

% retrieve equation residuals and bounds
apm_get(server,app,'apm_eqn.txt');
load apm_eqn.txt
delete('apm_eqn.txt');
y.neqn = size(apm_eqn,1); % get number of equations
y.eqn.lb = apm_eqn(:,1);
y.eqn.res = apm_eqn(:,2);
y.eqn.ub = apm_eqn(:,3);

% retrieve jacobian
apm_get(server,app,'apm_jac.txt');
load apm_jac.txt
delete('apm_jac.txt');
jac = apm_jac;
y.jac = sparse(jac(:,1),jac(:,2),jac(:,3),y.neqn,y.nvar);

% retrieve lagrange multipliers
apm_get(server,app,'apm_lam.txt');
load apm_lam.txt
delete('apm_lam.txt');
y.lam = apm_lam;

% retrieve hessian of the equations and objective
apm_get(server,app,'apm_hes.txt');
load apm_hes.txt
delete('apm_hes.txt');
hs = apm_hes;
y.hes = sparse(hs(:,1),hs(:,2),hs(:,3),y.nvar,y.nvar);

% retrieve hessian of the objective only
apm_get(server,app,'apm_hes_obj.txt');
load apm_hes_obj.txt
delete('apm_hes_obj.txt');
hs = apm_hes_obj;
y.hes_obj = sparse(hs(:,1),hs(:,2),hs(:,3),y.nvar,y.nvar);

% retrieve hessian of the equations only
apm_get(server,app,'apm_hes_eqn.txt');
load apm_hes_eqn.txt
delete('apm_hes_eqn.txt');
hs = apm_hes_eqn;
nhs = size(apm_hes_eqn,1);
i1(1:nhs)=0;
i2(1:nhs)=0;
% equations listed in sequential order
for i = 1:nhs,
    if(i1(hs(i,1))==0),
        i1(hs(i,1)) = i;
    end
    i2(hs(i,1)) = i;
end
for i = 1:y.neqn,
    if (i1(i)==0),
        y.hes_eqn{i} = [];
    else
        y.hes_eqn{i} = sparse(hs(i1(i):i2(i),2),hs(i1(i):i2(i),3),hs(i1(i):i2(i),4),y.nvar,y.nvar);
    end
end

% restore values if they exist, if not max_iter = -999999
if (max_iter>-1),
   apm_option(server,app,'nlc.max_iter',max_iter);
   apm_option(server,app,'nlc.diaglevel',diaglevel);
end
